@@ -1,7 +1,7 @@
 /*
  * @(#)Midi.h 3.00 14 May 1999
  *
- * Copyright (c) 2000 Pete Goodliffe (pete.goodliffe@pace.co.uk)
+ * Copyright (c) 2000 Pete Goodliffe (pete@cthree.org)
  *
  * This file is part of TSE3 - the Trax Sequencer Engine version 3.00.
  *
@@ -528,34 +528,34 @@ namespace TSE3
          *
          * @param status  Status value (4 bits).
          * @param channel Channel value (0-15).
-         * @param port    Port value (0-4).
+         * @param port    Port value.
          * @param data1   First data byte value (0-127).
          * @param data2   Second data byte value (0-127).
          */
         MidiCommand(int status, int channel, int port,
                     int data1, int data2)
-            : port(port), selected(0), data2(data2), data1(data1),
-              status(status), channel(channel) {}
+            : port(port), status(status), channel(channel),
+              data1(data1), data2(data2), selected(0) {}
 
         /**
          * Construct a MidiCommmand with one data byte
          *
          * @param status  Status value (4 bits).
          * @param channel Channel value (0-15).
-         * @param port    Port value (0-3).
+         * @param port    Port value.
          * @param data1   Data byte value (0-127).
          */
         MidiCommand(int status, int channel, int port,
                     int data1)
-            : port(port), selected(0), data2(0), data1(data1),
-              status(status), channel(channel) {}
+            : port(port), status(status), channel(channel),
+              data1(data1), data2(0), selected(0) {}
 
         /**
          * Construct a MidiCommand with status MidiCommand_Invalid.
          */
         MidiCommand()
-            : port(0), selected(0), data2(0), data1(0),
-              status(MidiCommand_Invalid), channel(0) {}
+            : port(0), status(MidiCommand_Invalid), channel(0),
+              data1(0), data2(0), selected(0) {}
 
         /**
          * Returns the number of data bytes this type of MidiCommand has.
@@ -579,12 +579,42 @@ namespace TSE3
         }
 
         /**
-         * The port value (0-3). This is defined as 0-3 simply becuase that's
-         * what the Acorn MIDI cards provide. Maybe I'll change this in the
-         * future.
+         * Returns whether the data1 value is defined to be a note number.
+         *
+         * @return Whether data1 contains a note value
          */
-        unsigned port     : 4;
-        unsigned          : 3;
+        bool isNote() const
+        {
+            return (status >= MidiCommand_NoteOff
+                    && status <= MidiCommand_KeyPressure);
+        }
+
+        /**
+         * The port value. Valid port numbers are positive integers.
+         * There are some defined minus numbers that have special
+         * meanings, too.
+         */
+        int port;
+
+        /**
+         * The status (type of MIDI message identifier) value (4 bits).
+         */
+        unsigned status   : 4;
+
+        /**
+         * The channel value (0-15, with some magic minus numbers).
+         */
+        signed channel  : 5;
+
+        /**
+         * The data1 value (0-127).
+         */
+        unsigned data1    : 8;
+
+        /**
+         * The data2 value (0-127).
+         */
+        unsigned data2    : 8;
 
         /**
          * Whether this command is 'selected' or not (0/1). This may be used
@@ -594,24 +624,26 @@ namespace TSE3
         unsigned selected : 1;
 
         /**
-         * The data2 value (0-127).
+         * These are some "reserved" port numbers. Valid port numbers
+         * are positive integers. Negative integers have specific meanings.
          */
-        unsigned data2    : 8;
+        enum MagicPortNumbers
+        {
+            NoPort   = -1,
+            AllPorts = -2,
+            SamePort = -3
+        };
 
         /**
-         * The data1 value (0-127).
+         * These are some "reserved" channel numbers. Valid channel numbers
+         * are 0-15. Negative integers have specific meanings.
          */
-        unsigned data1    : 8;
-
-        /**
-         * The status (type of MIDI message identifier) value (4 bits).
-         */
-        unsigned status   : 4;
-
-        /**
-         * The channel value (0-15).
-         */
-        unsigned channel  : 4;
+        enum MagicChannelNumbers
+        {
+            NoChannel   = -1,
+            AllChannels = -2,
+            SameChannel = -3
+        };
 
         /**
          * MidiCommands are considered equal if all data members except
