@@ -1,7 +1,7 @@
 /*
  * @(#)Notifier.h 3.00 13 July 2000
  *
- * Copyright (c) 2000 Pete Goodliffe (pete.goodliffe@pace.co.uk)
+ * Copyright (c) 2000 Pete Goodliffe (pete@cthree.org)
  *
  * This file is part of TSE3 - the Trax Sequencer Engine version 3.00.
  *
@@ -232,7 +232,7 @@ namespace TSE3
                                                 + arg_count<p4_type>::count;
                                                                          // (*)
                     void_list copy_list(listeners);
-                    for (unsigned int i = 0; i < copy_list.size(); i++)
+                    for (unsigned int i = 0; i < copy_list.size(); ++i)
                     {
                         if (listeners.contains(copy_list[i]))
                         {
@@ -467,7 +467,7 @@ namespace TSE3
             virtual ~Notifier()
             {
                 // LOCK
-                for (unsigned int i = 0; i < listeners.size(); i++)
+                for (unsigned int i = 0; i < listeners.size(); ++i)
                 {
                     listener_type *l
                         = static_cast<listener_type*>(listeners[i]);
@@ -648,7 +648,7 @@ namespace TSE3
             virtual ~Listener()
             {
                 // LOCK
-                for (unsigned int i = 0; i < notifiers.size(); i++)
+                for (unsigned int i = 0; i < notifiers.size(); ++i)
                 {
                     notifier_type *n
                         = static_cast<notifier_type*>(notifiers[i]);
