@@ -1,7 +1,7 @@
 /*
  * @(#)tse3play.cpp 3.00 30 July 1999
  *
- * Copyright (c) 2000 Pete Goodliffe (pete.goodliffe@pace.co.uk)
+ * Copyright (c) 2000 Pete Goodliffe (pete@cthree.org)
  *
  * This file is part of TSE3 - the Trax Sequencer Engine version 3.00.
  *
@@ -18,7 +18,7 @@
 
 #include "tse3play/tse3play.h"
 
-#include "tse3/plt/Unix.h"
+#include "tse3/plt/Factory.h"
 #include "tse3/util/MidiScheduler.h"
 #include "tse3/TSE3MDL.h"
 #include "tse3/TSE2MDL.h"
@@ -63,6 +63,10 @@ namespace
      * move cursor, change colour, etc.
      */
     const char escChar = char(0x1b);
+
+    const int USE_INTERNAL_PORT    = -100;
+    const int USE_EXTERNAL_PORT    = -101;
+    const int USE_BEST_EFFORT_PORT = -101;
 }
 
 /*****************************************************************************
@@ -72,14 +76,13 @@ namespace
 const char *TSE3Play::_name = "tse3play";
 
 TSE3Play::TSE3Play(int argc, char *argv[])
-: verbose(0), doplay(true), dovisual(true), doloop(false),
+: verbose(0), listports(false), doplay(true), dovisual(true), doloop(false),
   outmidiformat(1), outmidicompact(false),
-  metronome(false),
+  dometronome(false),
   midi(false), gm(false), gs(false), xg(false),
   startClock(-1), soloTrack(-1), usleepPeriod(100),
   dostop(true), doecho(false),
-  schedtype(Unix), patchesDir(""), port(-1), fastMidi(false),
-  sch(NULL)
+  schedtype(Unix), patchesDir(""), port(USE_BEST_EFFORT_PORT), fastMidi(false)
 {
     // The list of switches understood by this program
 
@@ -90,6 +93,9 @@ TSE3Play::TSE3Play(int argc, char *argv[])
     switches.push_back(Switch("verbose", "v", 0,
                               "give verbose progress information",
                                &TSE3Play::handle_verbose));
+    switches.push_back(Switch("list-ports", "list", 0,
+                              "list available port numbers",
+                              &TSE3Play::handle_listports));
     switches.push_back(Switch("novisual", "nv", 0,
                               "don't produce visual playback feedback",
                               &TSE3Play::handle_novisual));
@@ -98,17 +104,17 @@ TSE3Play::TSE3Play(int argc, char *argv[])
     switches.push_back(Switch("loop", "l", 0, "set loop mode",
                               &TSE3Play::handle_loop));
     switches.push_back(Switch("out-midi", "omidi", 1,
-                              "convert file to MIDI (filename follows switch)",
+                              "convert file to MIDI\n(filename follows switch)",
                               &TSE3Play::handle_outmidi));
     switches.push_back(Switch("out-midi-format-0", "omidi0", 1,
-                              "specify MIDI file format as 0 (default is 1)",
+                              "specify MIDI file format as 0\n(default is 1)",
                               &TSE3Play::handle_outmidiformat0));
     switches.push_back(Switch("out-midi-compact", "omidicomp", 1,
                               "specify MIDI file compaction option",
                               &TSE3Play::handle_outmidicompact));
     switches.push_back(Switch("out-tse3mdl", "otse3mdl", 1,
-                              "convert file to tse3mdl"
-                              " (filename follows switch)",
+                              "convert file to tse3mdl\n"
+                              "(filename follows switch)",
                               &TSE3Play::handle_outtse3mdl));
     switches.push_back(Switch("map-channel", "map", 2,
                               "<f> <t>: map channel f to t",
@@ -132,21 +138,15 @@ TSE3Play::TSE3Play(int argc, char *argv[])
                               "selects the best Unix scheduler device for MIDI "
                               "output (default)",
                               &TSE3Play::handle_unix));
-#ifdef TSE3_WITH_OSS
     switches.push_back(Switch("oss-scheduler", "oss", 0,
                               "selects the OSS for MIDI output",
                               &TSE3Play::handle_oss));
-#endif
-#ifdef TSE3_WITH_ALSA
     switches.push_back(Switch("alsa-scheduler", "alsa", 0,
                               "selects the ALSA library for MIDI output",
                               &TSE3Play::handle_alsa));
-#endif
-#ifdef TSE3_WITH_ARTS
     switches.push_back(Switch("arts-scheduler", "arts", 0,
                               "selects aRts for MIDI output",
                               &TSE3Play::handle_arts));
-#endif
     switches.push_back(Switch("stream-scheduler", "stream", 0,
                               "selects standard output stream for MIDI output",
                               &TSE3Play::handle_stream));
@@ -157,7 +157,7 @@ TSE3Play::TSE3Play(int argc, char *argv[])
                               "play with solo on given input (not MIDI) track",
                               &TSE3Play::handle_solo));
     switches.push_back(Switch("sleep", "sl", 1,
-                              "sets OS usecs sleep time between updates "
+                              "sets OS usecs sleep time between updates\n"
                               "(default 100)",
                               &TSE3Play::handle_sleep));
     switches.push_back(Switch("no-stop", "stop", 0,
@@ -170,12 +170,16 @@ TSE3Play::TSE3Play(int argc, char *argv[])
     switches.push_back(Switch("mute-track", "mute", 1,
                               "mutes the specified track (first is 0)",
                               &TSE3Play::handle_mute));
-#ifdef TSE3_WITH_OSS
     switches.push_back(Switch("patches-dir", "pd", 1,
                               "specifies the directory for patch files "
                               "(OSS only)",
                               &TSE3Play::handle_patchesdir));
-#endif
+    switches.push_back(Switch("internal", "int", 0,
+                              "force all output to first internal port",
+                              &TSE3Play::handle_internal));
+    switches.push_back(Switch("external", "ext", 0,
+                              "force all output to first external port",
+                              &TSE3Play::handle_internal));
     switches.push_back(Switch("force-port", "port", 1,
                               "force all output to be on specified port",
                               &TSE3Play::handle_port));
@@ -241,9 +245,6 @@ TSE3Play::~TSE3Play()
 
 int TSE3Play::go()
 {
-    Playable *playable = 0;
-    Song     *song     = 0;
-
     if (verbose)
     {
         cout << "\n*** tse3play (verbose mode) ***\n\n";
@@ -251,81 +252,147 @@ int TSE3Play::go()
         cout << "\n";
     }
 
-    // Create the scheduler and other playback objects
-    if (patchesDir != std::string())
+    Playable      *playable  = 0;
+    Song          *song      = 0;
+    MidiScheduler *sch       = 0;
+    Metronome     *metronome = 0;
+    Transport     *transport = 0;
+
+    if (doplay || midi || gm || gs || xg)
     {
-#ifdef TSE3_WITH_OSS
-        Plt::OSSMidiScheduler::setFmPatchesDirectory(patchesDir);
-        Plt::OSSMidiScheduler::setGusPatchesDirectory(patchesDir);
-#else
-        cout << "Patch dir specificed, but no OSS scheduler support\n";
-#endif
-    }
-    if (schedtype == Unix)
-    {
-        Plt::UnixMidiSchedulerFactory msf;
-        sch = msf.createScheduler();
-    }
-#ifdef TSE3_WITH_OSS
-    else if (schedtype == OSS)
-    {
-        Plt::OSSMidiSchedulerFactory msf;
-        sch = msf.createScheduler();
-    }
-#endif
-#ifdef TSE3_WITH_ALSA
-    else if (schedtype == Alsa)
-    {
-        Plt::AlsaMidiSchedulerFactory msf;
-        sch = msf.createScheduler();
-    }
-#endif
-#ifdef TSE3_WITH_ARTS
-    else if (schedtype == Arts)
-    {
-        Plt::ArtsMidiSchedulerFactory msf;
-        sch = msf.createScheduler();
-    }
-#endif
-    else
-    {
-        sch = new Util::StreamMidiScheduler();
-    }
-    if (verbose)
-    {
-        cout << "MidiScheduler details follow\n"
-             << "\n  Implementation name: " << sch->implementationName()
-             << "\n             No ports: " << sch->ports();
-        for (size_t port = 0; port < sch->ports(); port++)
+        if (patchesDir != std::string())
         {
-            cout << "\n  ------- Port number: " << port
-                 << "\n                 Type: " << sch->portType(port)
-                 << "\n                 Name: " << sch->portName(port)
-                 << "\n          Is readable: ";
-            if (sch->portReadable(port)) cout << "Yes"; else cout << "No";
-            cout << "\n         Is writeable: ";
-            if (sch->portWriteable(port)) cout << "Yes"; else cout << "No";
+#ifdef TSE3_WITH_OSS
+            Plt::OSSMidiScheduler::setFmPatchesDirectory(patchesDir);
+            Plt::OSSMidiScheduler::setGusPatchesDirectory(patchesDir);
+#else
+            cout << "Patch dir specificed, but no OSS scheduler support\n";
+#endif
         }
-        cout << "\n\n";
+
+        if (schedtype == Stream)
+        {
+            sch = new Util::StreamMidiScheduler();
+        }
+        else
+        {
+            using namespace Plt::UnixMidiSchedulerFactory;
+            switch (schedtype)
+            {
+                case Arts:
+                {
+#ifdef TSE3_WITH_ARTS
+                    setPreferredPlatform(UnixPlatform_Arts);
+                    break;
+#endif
+                }
+                case Alsa:
+                {
+#ifdef TSE3_WITH_ALSA
+                    setPreferredPlatform(UnixPlatform_Alsa);
+                    break;
+#endif
+                }
+                case OSS:
+                {
+#ifdef TSE3_WITH_OSS
+                    setPreferredPlatform(UnixPlatform_OSS);
+                    break;
+#endif
+                }
+                case Unix:   // Whatever comes
+                case Stream: // Invalid case, but quietens warnings
+                {
+                    break;
+                }
+            }
+            MidiSchedulerFactory msf;
+            sch = msf.createScheduler();
+        }
+
+        if (listports || verbose)
+        {
+            cout << "MidiScheduler details follow\n"
+                 << "\n  Implementation name: " << sch->implementationName()
+                 << "\n            Num ports: " << sch->numPorts();
+            std::vector<int> portNums;
+            sch->portNumbers(portNums);
+            for (size_t port = 0; port < sch->numPorts(); port++)
+            {
+                cout << "\n  ------- Port number: " << portNums[port]
+                     << "\n                 Type: " << sch->portType(portNums[port])
+                     << "\n                 Name: " << sch->portName(portNums[port])
+                     << "\n          Is readable: ";
+                if (sch->portReadable(portNums[port]))
+                    cout << "Yes";
+                else
+                    cout << "No";
+                cout << "\n         Is writeable: ";
+                if (sch->portWriteable(portNums[port]))
+                    cout << "Yes";
+                else
+                    cout << "No";
+                cout << "\n         Destination:  ";
+                if (sch->portInternal(portNums[port]))
+                    cout << "Internal";
+                else
+                    cout << "External";
+            }
+            cout << "\n\n";
+        }
+
+        if (port == USE_INTERNAL_PORT)
+        {
+            port = sch->defaultInternalPort();
+            if (verbose && port == MidiCommand::NoPort)
+            {
+                cout << "No default internal port, no port forced\n";
+            }
+        }
+        else if (port == USE_EXTERNAL_PORT)
+        {
+            port = sch->defaultInternalPort();
+            if (verbose && port == MidiCommand::NoPort)
+            {
+                cout << "No default external port, no port forced\n";
+            }
+        }
+        else if (port == USE_BEST_EFFORT_PORT)
+        {
+            port = sch->defaultInternalPort();
+            if (port == MidiCommand::NoPort)
+            {
+                if (verbose) cout << "No default internal port\n";
+                port = sch->defaultExternalPort();
+                if (port == MidiCommand::NoPort && verbose)
+                    cout << "No default internal port\n";
+            }
+        }
+
+        attachTo(sch);
+
+        // Create the scheduler and other playback objects
+        metronome = new Metronome();
+        metronome->setStatus(Transport::Playing, dometronome);
+        transport = new Transport(metronome, sch);
+        setPanic(transport->startPanic());
+        setPanic(transport->endPanic());
+        transport->midiEcho()->filter()->setStatus(doecho);
     }
-    attachTo(sch);
-    Metronome m;
-    m.setStatus(Transport::Playing, metronome);
-    Transport *transport = new Transport(&m, sch);
-    setPanic(transport->startPanic());
-    setPanic(transport->endPanic());
-    transport->midiEcho()->filter()->setStatus(doecho);
 
     // Draw the on screen presence
 
+    bool dovisual_nodeferedframe
+        = dovisual && !verbose && outmidi != "-" && outtse3mdl != "-";
+
     TSE3PlayVisual visual(transport, sch);
-    if (dovisual && !verbose) visual.drawFrame();
+    if (dovisual_nodeferedframe) visual.drawFrame();
 
     // Loading the file
 
     if (filename != "")
     {
-        if (dovisual && !verbose) visual.message("Loading file");
+        if (dovisual_nodeferedframe) visual.message("Loading file");
         FileRecogniser fr(filename);
         if (verbose) cout << "File type is: ";
         switch (fr.type())
@@ -333,7 +400,7 @@ int TSE3Play::go()
             default:
             case FileRecogniser::Type_Error:
             {
-                if (dovisual && !verbose)
+                if (dovisual_nodeferedframe)
                     visual.message("File couldn't be opened. Exiting.");
                 if (verbose) cout << "load error - exiting\n";
                 cerr << "Load error for \"" << filename.c_str() << "\"\n";
@@ -341,7 +408,7 @@ int TSE3Play::go()
             }
             case FileRecogniser::Type_Unknown:
             {
-                if (dovisual && !verbose)
+                if (dovisual_nodeferedframe)
                     visual.message("Filetype unknown. Exiting.");
                 if (verbose) cout << "unknown - exiting\n";
                 cerr << "Unknown filetype for \"" << filename.c_str() << "\"\n";
@@ -349,7 +416,7 @@ int TSE3Play::go()
             }
             case FileRecogniser::Type_TSE3MDL:
             {
-                if (dovisual && !verbose)
+                if (dovisual_nodeferedframe)
                     visual.message("Loading TSE3MDL file");
                 if (verbose) cout << "TSE3MDL\n";
                 TSE3MDL tse3mdl(_name);
@@ -359,7 +426,7 @@ int TSE3Play::go()
             }
             case FileRecogniser::Type_TSE2MDL:
             {
-                if (dovisual && !verbose)
+                if (dovisual_nodeferedframe)
                     visual.message("Loading TSEMDL file");
                 if (verbose) cout << "TSEMDL (legacy TSE version 2)\n";
                 TSE2MDL fo(_name, verbose);
@@ -369,7 +436,7 @@ int TSE3Play::go()
             }
             case FileRecogniser::Type_Midi:
             {
-                if (dovisual && !verbose)
+                if (dovisual_nodeferedframe)
                     visual.message("Loading MIDI file");
                 if (verbose) cout << "MIDI\n";
                 MidiFileImport *mfi = new MidiFileImport(filename, verbose);
@@ -402,6 +469,7 @@ int TSE3Play::go()
         song     = new Song();
         playable = song;
     }
+
     if (dovisual)
     {
         // Don't need to do the calculation if it's not needed
@@ -412,42 +480,57 @@ int TSE3Play::go()
 
     if (outmidi != std::string() && song)
     {
-        if (dovisual && !verbose)
+        if (dovisual_nodeferedframe)
             visual.message("Performing MIDI output");
         MidiFileExport mfe(outmidiformat, outmidicompact, verbose);
-        mfe.save(outmidi.c_str(), song);
-    }
-
-    if (outtse3mdl != std::string() && song)
-    {
-        if (dovisual && !verbose)
-            visual.message("Performing TSE3MDL output");
-        TSE3MDL tse3mdl(_name);
-        tse3mdl.save(outtse3mdl, song);
-    }
-
-    // Perform modification of Song according to command line parameters
-
-    if (song)
-    {
-        song->setSoloTrack(soloTrack);
-        std::list<size_t>::iterator i = muteList.begin();
-        while (i != muteList.end())
+        if (outmidi == "-")
         {
-            if (*i <= song->size())
-                (*song)[*i]->filter()->setStatus(false);
-            i++;
+            mfe.save(std::cout, song);
+        }
+        else
+        {
+            mfe.save(outmidi, song);
+        }
+    }
+    if (outtse3mdl != std::string() && song)
+    {
+        if (dovisual_nodeferedframe)
+            visual.message("Performing TSE3MDL output");
+        TSE3MDL tse3mdl(_name);
+        if (outtse3mdl == "-")
+        {
+            tse3mdl.save(std::cout, song);
+        }
+        else
+        {
+            tse3mdl.save(outtse3mdl, song);
         }
     }
-    transport->filter()->setPort(port);
 
-    // Handle playback facilities
-
-    if (dovisual && verbose)  visual.drawFrame();
-    if (dovisual)             visual.blankFrame();
+    // Now do playback
 
     if (doplay)
     {
+        // Perform modification of Song according to command line parameters
+
+        if (song)
+        {
+            song->setSoloTrack(soloTrack);
+            std::list<size_t>::iterator i = muteList.begin();
+            while (i != muteList.end())
+            {
+                if (*i <= song->size())
+                    (*song)[*i]->filter()->setStatus(false);
+                i++;
+            }
+        }
+        transport->filter()->setPort(port);
+
+        // Handle playback facilities
+
+        if (dovisual && !dovisual_nodeferedframe) visual.drawFrame();
+        if (dovisual)                             visual.blankFrame();
+
         if (verbose && !dovisual) cout << "Playing...\n";
 
         more_to_come = true;
@@ -503,8 +586,9 @@ int TSE3Play::go()
 
     delete playable;
     delete transport;
+    delete metronome;
     delete sch;
-    sch = 0;
+
     return 0;
 }
 
@@ -543,7 +627,7 @@ void TSE3Play::handle_help(int, char*[])
              << "\n";
     }
 
-    cout << "\nSend bug reports to <pete.goodliffe@pace.co.uk>\n";
+    cout << "\nSend bug reports to <pete@cthree.org>\n";
     exit(0);
 }
 
@@ -563,6 +647,12 @@ void TSE3Play::handle_verbose(int, char*[])
 }
 
 
+void TSE3Play::handle_listports(int, char*[])
+{
+    listports = true;
+}
+
+
 void TSE3Play::handle_novisual(int, char*[])
 {
     dovisual = false;
@@ -623,7 +713,7 @@ void TSE3Play::handle_mapchannel(int argpos, char *argv[])
 
 void TSE3Play::handle_metronome(int, char*[])
 {
-    metronome = true;
+    dometronome = true;
     if (verbose) cout << "Enabling metronome\n";
 }
 
@@ -665,22 +755,34 @@ void TSE3Play::handle_unix(int, char*[])
 
 void TSE3Play::handle_oss(int, char*[])
 {
+#ifdef TSE3_WITH_OSS
     schedtype = OSS;
     if (verbose) cout << "Selecting OSS for MIDI output\n";
+#else
+    std::cerr << "OSS is not supported on this computer\n";
+#endif
 }
 
 
 void TSE3Play::handle_alsa(int, char*[])
 {
+#ifdef TSE3_WITH_ALSA
     schedtype = Alsa;
     if (verbose) cout << "Selecting ALSA for MIDI output\n";
+#else
+    std::cerr << "ALSA is not supported on this computer\n";
+#endif
 }
 
 
 void TSE3Play::handle_arts(int, char*[])
 {
+#ifdef TSE3_WITH_ARTS
     schedtype = Arts;
     if (verbose) cout << "Selecting aRts for MIDI output\n";
+#else
+    std::cerr << "aRts is not supported on this computer\n";
+#endif
 }
 
 
@@ -737,8 +839,26 @@ void TSE3Play::handle_mute(int argpos, char *argv[])
 
 void TSE3Play::handle_patchesdir(int argpos, char *argv[])
 {
+#ifdef TSE3_WITH_OSS
     patchesDir = argv[argpos+1];
     if (verbose) cout << "Patches directory is: " << patchesDir.c_str() << "\n";
+#else
+    std::cerr << "OSS is not supported on this computer\n";
+#endif
+}
+
+
+void TSE3Play::handle_internal(int, char *[])
+{
+    if (verbose) cout << "Forcing output to first internal port\n";
+    port = USE_INTERNAL_PORT;
+}
+
+
+void TSE3Play::handle_external(int, char *[])
+{
+    if (verbose) cout << "Forcing output to first external port\n";
+    port = USE_EXTERNAL_PORT;
 }
 
 
@@ -777,6 +897,20 @@ void TSE3Play::setPanic(Panic *panic)
 }
 
 
+TSE3Play::Switch::Switch
+    (std::string l, std::string s, int n, std::string h, handler_t hd)
+: lng(l), srt(s), nargs(n), help(h), handler(hd)
+{
+    for (std::string::size_type n = 0; n < help.size(); ++n)
+    {
+        if (help[n] == '\n')
+        {
+            help.replace(n, 1, "\n                                     ");
+        }
+    }
+}
+
+
 /*****************************************************************************
  * TSE3PlayVisual class
  ****************************************************************************/
@@ -789,8 +923,8 @@ TSE3PlayVisual::TSE3PlayVisual(Transport *t, MidiScheduler *s)
     {
         now[chan] = next[chan] = 0;
     }
-    transport->attachCallback(this);
-    attachTo(transport);
+    if (transport) transport->attachCallback(this);
+    if (transport) attachTo(transport);
 }
 
 
@@ -833,7 +967,7 @@ void TSE3PlayVisual::drawFrame()
          << "                               \n"
          << "           tse3play            \n"
          << "                               \n"
-         << "    (c) 2000 Pete Goodliffe    \n"
+         << "   (c) 2000-2 Pete Goodliffe   \n"
          << " Incorporates TSE3 technology  \n"
          << escChar << "[34m";
     for (int n = 5; n <= max; n++)
